#!/bin/bash
#===============================================================================
#   weather.sh      |   FreeBSD License     |   Version 1.0     |   2014-01-24
#   James Hendrie   |   hendrie dot james at gmail dot com
#
#   Description:    This is a script that tells you what the temperature is
#===============================================================================

##  Fairbanks, AK.  Where I live.
DEFAULT_ZIP=99701

##  If the user is too lazy to specify a zip code, use the default zip
ZIP=$DEFAULT_ZIP

##  Our temporary file
FILE=$(mktemp)

##  Whether or not we're using metric
METRIC=0

##  Whether or not to print temperature, place and time last updated
PTEMP=1
PPLACE=1
PUPDATE=0

##  If this is 1, print only the number
NUM_ONLY=0


##  If they want help
function print_help
{
    echo -e "Usage:\tweather [ARGS]\n"
    echo -e "\t-h\t\tThis help text"
    echo -e "\t-v\t\tVersion and author info"
    echo -e "\t-z [NUM]\tZip code"
    echo -e "\t-m\t\tUse metric units (Celsius)"
    echo -e "\t-t\t\tPrint only temperature"
    echo -e "\t-l\t\tPrint only the location"
    echo -e "\t-u\t\tPrint last updated"
    echo -e "\t-n\t\tPrint only the number (no location, no last updated, etc."
    echo -e "\t-c\t\tCreate ~/.weather file for customized default variables."
    echo -e "\t\t\tOther arguments will override these variables if used."

    echo -e "\nThis script tells you what the temperature is.  That's all."

}


##  Source the ~/.weather file if it exists
if [ -f "$HOME/.weather" ]; then
    source "$HOME/.weather"
fi


##  If they want a version number
function print_version
{
    echo "Version 1.0"
    echo "James Hendrie < hendrie dot james at gmail dot com > "
}


##  Create ~/.weather file
function create_weather_file
{
    WF="$HOME/.weather"
    touch "$WF"

    if [ $? -ne 0 ]; then
        echo "ERROR:  Cannot create file $WF" 1>&2
        exit 1
    fi

    echo "# This is the weather rc file.  In here, you can alter these" > "$WF"
    echo "# default values however you'd like." >> "$WF"

    echo "" >> "$WF"
    echo "# Default zip code" >> "$WF"
    echo "ZIP=99701" >> "$WF"

    echo "" >> "$WF"
    echo "# Whether or not to use metric (celsius)" >> "$WF"
    echo "METRIC=0" >> "$WF"

    echo "" >> "$WF"
    echo "# Whether or not to print the temperature" >> "$WF"
    echo "PTEMP=1" >> "$WF"

    echo "" >> "$WF"
    echo "# Whether or not to print the location" >> "$WF"
    echo "PPLACE=1" >> "$WF"

    echo "" >> "$WF"
    echo "# Whether or not to print the time last updated" >> "$WF"
    echo "PUPDATE=0" >> "$WF"

    echo "" >> "$WF"
    echo "# Whether or not to print only the number (no location, etc)" >> "$WF"
    echo "NUM_ONLY=0" >> "$WF"

}



##  Argument handling
while getopts "hvmnclutz:" flag; do
    if [ "$flag" = "h" ]; then
        print_help
        exit 0
    elif [ "$flag" = "v" ]; then
        print_version
        exit 0
    elif [ "$flag" = "c" ]; then
        create_weather_file
        exit 0
    elif [ "$flag" = "m" ]; then
        let METRIC=1
    elif [ "$flag" = "n" ]; then
        let PPLACE=0
        let PUPDATE=0
        let NUM_ONLY=1
    elif [ "$flag" = "t" ]; then
        let PPLACE=0
        let PUPDATE=0
    elif [ "$flag" = "l" ]; then
        let PTEMP=0
        let PUPDATE=0
    elif [ "$flag" = "u" ]; then
        let PUPDATE=1
    elif [ "$flag" = "z" ]; then
        let ZIP="$OPTARG"
    fi
done



#=========================      MAIN    =======================================
curl -s "http://www.weather.com/weather/today/$ZIP" > $FILE

PLACE=$(cat "$FILE" | grep $ZIP | grep \+ | head -n1 \
    | cut -f4 -d'"' | cut -f4 -d'/' | sed 's/\+/\ /g' | cut -f1 -d':')

LAST_UPDATED=$(cat "$FILE" | grep 'last-updated' | head -n1 | cut -f2 -d'>'\
    | cut -f1 -d'<')

TEMPERATURE=$(cat "$FILE" | grep 'temperature-fahrenheit' | head -n1\
    | cut -f2 -d'>' | cut -f1 -d'<')

##  Check for metric conversion
if [ "$METRIC" -eq 1 ]; then

    r1=$(( $TEMPERATURE - 32 ))

    ##  Check for presence of 'bc'
    which bc &> /dev/null
    if [ $? -ne 0 ]; then
        echo "ERROR:  Program 'bc' not found; results will not be accurate" 1>&2
        TEMPERATURE=$(( $r1 / 2 ))
    else
        TEMPERATURE=$(echo "$r1 / 1.8" | bc -l | head -c4)
    fi
fi
#=========================      END MAIN =======================================


##
if [ "$NUM_ONLY" -eq 1 ]; then
    echo "$TEMPERATURE"
    exit 0
fi


##  Print the results as specified
if [ "$PPLACE" -eq 1 ]; then
    echo -n "Weather for $PLACE"

    if [ "$PUPDATE" -eq 1 ]; then
        echo -n " (last updated $LAST_UPDATED)"
    fi

    echo -e ":\n"
fi


if [ "$PTEMP" -eq 1 ]; then
    echo -n "$TEMPERATURE"

    if [ "$NUM_ONLY" -ne 1 ]; then
        if [ "$METRIC" -eq 1 ]; then
            echo -n " C"
        else
            echo -n " F"
        fi
    fi

    echo ""
fi

rm "$FILE"
